﻿Imports guinet
Imports System.Windows.Forms
Imports System.Data
Imports System.Drawing
Imports System.Windows.Forms.DataVisualization.Charting



Public Class zcso


#Region "Private data"
    ' interface to GuiXT
    Private myguixt As guixt

    ' Dictionary for sales per product and month
    Private matdict1 As New Dictionary(Of String, Decimal)

    ' Dictionary for sales per product
    Private matdict2 As New Dictionary(Of String, Decimal)

    ' Dictionary for product names
    Private matdict3 As New Dictionary(Of String, String)

#End Region
   

    ' embed into SAP GUI window
    Public Sub embed(ByVal w As Object)
        guixt.EmbedForm(Me, w)
    End Sub

    ' initialize for new customer data 
    Public Sub initialize()

        ' create guixt object for communication with GuiXT
        myguixt = New guixt

        ' clear data grids
        contacts.RowCount = 0
        orders.RowCount = 0
        matbuttons.RowCount = 0

        ' clear product dictionaries
        matdict1.Clear()
        matdict2.Clear()
        matdict3.Clear()

    End Sub

    ' final display, all customer data are read
    Public Sub display()

        ' sort contacts
        contacts.Sort(contacts.Columns(0), System.ComponentModel.ListSortDirection.Ascending)

        ' sort orders
        orders.Sort(orders.Columns(0), System.ComponentModel.ListSortDirection.Descending)


        ' Build Sales chart
        GenerateSalesChart("*")

        ' no selection
        contacts.ClearSelection()
        orders.ClearSelection()


    End Sub

    ' Pass all product numbers to GuiXT via GuiXT long text variable
    Public Sub products_to_guixt()

        ' Pass product numbers back to GuiXT to obtain product names
        myguixt.SetText("zcso_products", String.Join(vbCrLf, matdict2.Keys))

    End Sub

    ' Obtain all product texts from GuiXT via GuiXT long text variables
    Public Sub productnames_from_guixt()

        Dim products As String() = myguixt.GetText("r1").Split(vbCrLf)
        Dim productnames As String() = myguixt.GetText("r2").Split(vbCrLf)

        For k As Integer = 0 To products.Length
            matdict3.Add(products(k).Trim, productnames(k).Trim)
        Next


    End Sub

#Region "Event handling"


    'Create new standard order
    Private Sub buttoncreateorder_Click(sender As Object, e As EventArgs) Handles buttoncreateorder.Click
        myguixt.Input("OK:/0,process=zcso_create_order.txt")
    End Sub

    'Create new order, promotion 1
    Private Sub promotion1_Click(sender As Object, e As EventArgs) Handles promotion1.Click
        myguixt.Input("U[PROMOTION]:1;OK:/0,process=zcso_create_order.txt")
    End Sub

    'Create new order, promotion 2 
    Private Sub promotion2_Click(sender As Object, e As EventArgs) Handles promotion2.Click
        myguixt.Input("U[PROMOTION]:2;OK:/0,process=zcso_create_order.txt")
    End Sub

    'Create new order, promotion 3
    Private Sub promotion3_Click(sender As Object, e As EventArgs) Handles promotion3.Click
        myguixt.Input("U[PROMOTION]:3;OK:/0,process=zcso_create_order.txt")
    End Sub

    ' Display customer order
    Private Sub buttondisplayyorder_Click(sender As Object, e As EventArgs) Handles buttondisplayorder.Click
        Dim ordernumber As String = ""
        For Each row In orders.SelectedRows
            ordernumber = row.Cells("ordervbeln").Value()
        Next

        myguixt.SetVariable("zcso_vbeln", ordernumber)
        myguixt.Input("OK:/0,process=zcso_display_order.txt")
    End Sub

    ' Change customer order
    Private Sub buttonchangeorder_Click(sender As Object, e As EventArgs) Handles buttonchangeorder.Click

        Dim ordernumber As String = ""
        For Each row In orders.SelectedRows
            ordernumber = row.Cells("ordervbeln").Value()
        Next
        myguixt.SetVariable("zcso_vbeln", ordernumber)
        myguixt.Input("OK:/0,process=zcso_change_order.txt")
    End Sub

    ' Click on product button to generate chart
    Private Sub matbuttons_CellClick(sender As Object, e As DataGridViewCellEventArgs) Handles matbuttons.CellClick

        Dim product As String = ""
        For Each row In matbuttons.SelectedRows
            product = row.Cells("prodnumber").Value()
        Next

        GenerateSalesChart(product)

    End Sub
#End Region


#Region "contacts"

    ' called from GuiXT for each new contact
    Public Sub newcontact()

        Dim k As Integer = contacts.Rows.Add()

        Dim c As System.Windows.Forms.DataGridViewCellCollection = contacts.Rows.Item(k).Cells

        c("contactname").Value = myguixt.GetVariable("zcso_contactname")
        c("contacttitle").Value = myguixt.GetVariable("zcso_contacttitle")
        c("contactphone").Value = myguixt.GetVariable("zcso_contactphone")
        c("contactemail").Value = myguixt.GetVariable("zcso_contactemail")

    End Sub



#End Region

#Region "orders"

    ' transfer order data from GuiXT to datagrid
    Public Sub neworders()

        Dim r1 As String() = myguixt.GetText("r1").Split(vbCrLf)
        Dim r2 As String() = myguixt.GetText("r2").Split(vbCrLf)
        Dim r3 As String() = myguixt.GetText("r3").Split(vbCrLf)
        Dim r4 As String() = myguixt.GetText("r4").Split(vbCrLf)
        Dim r5 As String() = myguixt.GetText("r5").Split(vbCrLf)


        For k As Integer = 0 To r1.Length - 1

            orders.Rows.Add()

            Dim o As System.Windows.Forms.DataGridViewCellCollection = orders.Rows.Item(k).Cells

            o("ordervbeln").Value = CInt(r1(k).Trim)
            o("orderaudat").Value = r2(k).Trim
            o("orderauart").Value = r3(k).Trim
            o("ordernetwr").Value = Decimal.Parse(r4(k).Trim, Globalization.NumberStyles.AllowDecimalPoint).ToString()
            o("orderbstnk").Value = r5(k).Trim

        Next
    End Sub


#End Region


#Region "materials"

    Public Sub materialsales()

        Dim r1 As String() = myguixt.GetText("r1").Split(vbCrLf)
        Dim r2 As String() = myguixt.GetText("r2").Split(vbCrLf)
        Dim r3 As String() = myguixt.GetText("r3").Split(vbCrLf)

        For k As Integer = 0 To r1.Length - 1

            Dim product As String = r1(k).Trim
            Dim year As String = r2(k).Trim.Substring(0, 4)
            Dim netwr As Decimal = Decimal.Parse(r3(k).Trim)


            Dim key As String = year & product

            If matdict1.ContainsKey(key) Then
                matdict1.Item(key) = matdict1.Item(key) + netwr
            Else
                matdict1.Add(key, netwr)
            End If


            ' product only
            If matdict2.ContainsKey(product) Then
                matdict2.Item(product) = matdict2.Item(product) + netwr
            Else
                matdict2.Add(product, netwr)
            End If

        Next




    End Sub



#End Region


#Region "charts"

    Private total_netwr0 As Decimal = 0
    Private total_netwr1 As Decimal = 0
    Private total_netwr2 As Decimal = 0
    Private total_netwr3 As Decimal = 0
    Private total_netwr4 As Decimal = 0

    Public Sub GenerateSalesChart(product As String)

        ' create chart
        Dim Chart1 As New Chart()

        Chart1.Size = New System.Drawing.Size(salesdiagram.Size.Width, salesdiagram.Size.Height)
        Chart1.BackColor = Color.FromArgb(240, 240, 240)

        Dim ChartArea1 As New ChartArea

        ChartArea1.Name = "ChartArea1"
        Chart1.ChartAreas.Add(ChartArea1)

        Dim Series1 As New Series
        Series1.ChartArea = "ChartArea1"
        Series1.Palette = ChartColorPalette.Pastel
        Series1.XValueMember = "Year"
        Series1.YValueMembers = "Sales"
        Chart1.Series.Add(Series1)

        Dim table As New DataTable

        ' Create columns in the DataTable
        table.Columns.Add("Year", GetType(String))
        table.Columns.Add("Sales", GetType(Integer))

        Dim year0 As String = myguixt.GetVariable("zcso_year0")
        Dim year1 As String = myguixt.GetVariable("zcso_year1")
        Dim year2 As String = myguixt.GetVariable("zcso_year2")
        Dim year3 As String = myguixt.GetVariable("zcso_year3")
        Dim year4 As String = myguixt.GetVariable("zcso_year4")

        Dim netwr0 As Decimal = 0
        Dim netwr1 As Decimal = 0
        Dim netwr2 As Decimal = 0
        Dim netwr3 As Decimal = 0
        Dim netwr4 As Decimal = 0

        For Each m In matdict1

            If product = "*" Or product = "***" Or product = m.Key.Substring(4) Then
                Select Case m.Key.Substring(0, 4)

                    Case year0
                        netwr0 += m.Value
                    Case year1
                        netwr1 += m.Value
                    Case year2
                        netwr2 += m.Value
                    Case year3
                        netwr3 += m.Value
                    Case year4
                        netwr4 += m.Value

                End Select
            End If
        Next

        ' Save total values
        If product = "*" Then
            total_netwr0 = netwr0
            total_netwr1 = netwr1
            total_netwr2 = netwr2
            total_netwr3 = netwr3
            total_netwr4 = netwr4


            ' build matbuttons
            Dim total_netwr As Decimal = total_netwr0 + total_netwr1 + total_netwr2 + total_netwr3 + total_netwr4

          

            matbuttons.Rows.Add()

            Dim mb As System.Windows.Forms.DataGridViewCellCollection = matbuttons.Rows.Item(0).Cells

            mb("percentage").Value = "100"
            mb("product").Value = "***"
            mb("prodnumber").Value = "***"
            mb("prodname").Value = "All products"
            mb("netwr").Value = total_netwr

            ' select first row
            matbuttons.ClearSelection()
            matbuttons.Rows.Item(0).Selected = True


            For Each m In matdict2
                Dim k As Integer = matbuttons.Rows.Add()

                mb = matbuttons.Rows.Item(k).Cells


                Dim percentage As Integer
                ' avoid to divide by 0
                If total_netwr = 0 Then
                    percentage = 0
                Else
                    percentage = CInt(m.Value * 100 / total_netwr)
                End If



                mb("percentage").Value = percentage.ToString


                ' Suppress leading 000
                If IsNumeric(m.Key) Then
                    mb("product").Value = CInt(m.Key).ToString
                Else
                    mb("product").Value = m.Key
                End If

                mb("prodnumber").Value = m.Key
                mb("prodname").Value = matdict3.Item(m.Key)

                mb("netwr").Value = m.Value

            Next


            ' sortmatbuttons
            matbuttons.Sort(matbuttons.Columns("netwr"), System.ComponentModel.ListSortDirection.Descending)

        End If


        ' Sales figures for 5 years
        table.Rows.Add(year4, CInt(netwr4))
        table.Rows.Add(year3, CInt(netwr3))
        table.Rows.Add(year2, CInt(netwr2))
        table.Rows.Add(year1, CInt(netwr1))
        table.Rows.Add(year0, CInt(netwr0))

        ' Chart data
        Chart1.DataSource = table


        Dim bmp As New Bitmap(Chart1.Width, Chart1.Height)
        Dim rec As New Rectangle(0, 0, Chart1.Width, Chart1.Height)
        Chart1.DrawToBitmap(bmp, rec)

        salesdiagram.Image = bmp


        ' Title variable
        If product = "*" OrElse product = "***" Then
            product_selected.Text = "All products"
        Else
            product_selected.Text = product & " " & matdict3.Item(product)
        End If

        salesdiagram.Refresh()

    End Sub

  

#End Region

   
End Class